/* $Header: /cvsroot/libclc/libclc/src/clc_errno.c,v 1.11 2003/06/01 18:45:24 augestad Exp $
 *
 * Copyright (c) 2003, B. Augestad (boa@metasystems.no)
 * COPYING RESTRICTIONS APPLY, see COPYING file.
 */

#include <stdio.h>
#include <errno.h>
#include <string.h>

#include "clc_errno.h"

static const struct {
	int errnum;
	const char* msg;
} clc_errlist[] = {
	{ CLC_ENOSPC, "Not enough space" },
	{ CLC_ENOMEM, "Allocation of memory failed" }
};

int clc_iserror(int errnum)
{
	/* nelem is int to avoid warnings when compared to errnum */
	const int nelem = (int)(sizeof(clc_errlist)/sizeof(clc_errlist[0]));

	if(errnum >= CLC_EMIN && errnum < (CLC_EMIN + nelem))
		return 1;
	else
		return 0;
}

const char* clc_strerror(int errnum)
{
	const char* unknown = "Unknown error.";
	size_t i;
	const size_t nelem = sizeof(clc_errlist)/sizeof(clc_errlist[0]);

	if(!clc_iserror(errnum)) {
		const char* msg = strerror(errnum);
		return (msg != NULL) ? msg : unknown;
	}

	for(i = 0; i < nelem; i++) {
		if(errnum == clc_errlist[i].errnum)
			return clc_errlist[i].msg;
	}

	return unknown;
}

void clc_perror(const char* s)
{
	if(clc_iserror(errno)) {
		const char *format = "%s: %s\n";
		if(s == NULL || *s == '\0') {
			format = "%s%s\n";
			s = "";
		}

		fprintf(stderr, format, s, clc_strerror(errno));
	}
	else 
		perror(s);
}

