/* $Id: clc_strreplace.c,v 1.4 2003/04/28 15:50:22 hfuru Exp $ */
/*
 * Copyright (c) 2003 Hallvard B. Furuseth <h.b.furuseth@usit.uio.no>
 * COPYING RESTRICTIONS APPLY, see COPYING file.
 */

#include <errno.h>
#include <string.h>
#include <stdlib.h>
#include "clc_assert.h"
#include "clc_errno.h"
#include "clc_string.h"


char *
clc_strreplace(const char *src, const char *old, const char *new)
{
#	define SUBS_TOP 32
	const char *subs[SUBS_TOP], *p, *sub_start, *src_without_old;
	char *dest, *ret;
	size_t old_len, new_len, dest_size, tmp;
	int i;

	clc_assert_not_null(clc_strreplace, src);
	clc_assert_arg(clc_strreplace, old != NULL && *old != '\0');
	clc_assert_not_null(clc_strreplace, new);

	old_len = strlen(old);
	new_len = strlen(new);

	/* Find substrings and compute size of result */
	dest_size = 1;
	src_without_old = src;
	for (i = 0, p = src ;; p = sub_start + old_len) {
		sub_start = strstr(p, old);
		if (i < SUBS_TOP)
			subs[i++] = sub_start;
		if (sub_start == NULL)
			break;
		dest_size += new_len;
		if (dest_size < new_len) {
			errno = ERANGE;
			return NULL;
		}
		src_without_old += old_len;
	}
	tmp = (p - src_without_old) + strlen(p);
	dest_size += tmp;
	if (dest_size < tmp) {
		errno = ERANGE;
		return NULL;
	}

	/* Make result string */
	ret = malloc(dest_size);
	if (ret == NULL) {
		errno = CLC_ENOMEM;
	} else {
		dest = ret;
		for (i = 0;; ) {
			p = (i < SUBS_TOP ? subs[i++] : strstr(src, old));
			if (p == NULL)
				break;
			tmp = p - src;
			memcpy(dest, src, tmp);
			src += tmp + old_len;
			dest = clc_stpcpy(dest + tmp, new);
		}
		strcpy(dest, src);
	}

	return ret;
}
